-- /***********************************************
--   Bomb Sweeper Info
--  *********************/
--	Author			: Moongaze (Twisting Nether)
--	Description		: A clone of the game Minesweeper which comes packed on pretty much
--				  every PC now-a-days. The object is to find the hidden mines by searching
--				  with the clues given when you click a square. If a bomb is near the square,
--				  a number will appear. The more bombs near the square, the higher the number.
--				  To win, you must mark all bomb squares without revealing them.
--
--	Project Start		: October 23, 2006
--	Project Milestones:	: October 24, 2006	- version 0.5 ready
--				: October 25, 2006	- version 0.9 ready
--	Project End		: Octover 26, 2006	- version 1.0 finished
--	
--	Todo List		: - Localization?
--
--	Wish List for Version 2	: - Doubt there will be one. Maybe more skins.
--
-- /***********************************************

-- /***********************************************
--   Bomb Sweeper Constants
--  *********************/

local BS_HEIGHT_TITLEBAR	= 23;	-- Height of the title bar
local BS_HEIGHT_FOOTERBAR	= 23;	-- Height of the footer bar
local BS_MAXROWS		= 24;	-- Maximum rows allowed in the game (can support WAY more than this, but it will be hard to see)
local BS_MAXCOLUMNS		= 30;	-- Maximum columns allowed in the game (can support WAY more than this, but it will be hard to see)
local BS_MAXBOMBS = (BS_MAXROWS - 1)	-- Maximum bombs allowed to be place on grid (although no one would PLAY with this many)
	* (BS_MAXCOLUMNS - 1);

local BS_EMPTY			= 0;	-- Game grid square: empty data
local BS_BOMB			= 9;	-- Game grid square: bomb data
local BS_REVEALED		= 13;	-- Game grid square: revealed square
local BS_HIDDEN			= 14;	-- Game grid square: hidden / not revealed square
local BS_FLAGGED		= 15;	-- Game grid square: square is flagged for a bomb
local BS_UNKNOWN		= 16;	-- Game grid square: square is unknown, user marked it as such

local BS_ALL			= 0;	-- flag for BS_updateSkinColor: updates all parts
local BS_BORDER			= 1;	-- flag for BS_updateSkinColor: updates just the borders
local BS_BACKGROUND		= 2;	-- flag for BS_updateSkinColor: updates just the backgrounds

local ADDON_PATH =			-- path for the addon, for less typing.
"Interface\\AddOns\\BombSweeper"

-- /***********************************************
--   Bomb Sweeper Variables
--  *********************/

local BS_roundWidth		= 10;	 -- Game's current width
local BS_roundHeight		= 10;	 -- Game's current height
local BS_roundBombs		= 10;	 -- Game's current amount of bombs
local BS_roundOver		= false; -- Flag to say if game is over

local storedR			= 0;	 -- stores the original R value of the background/border before changing it
local storedG			= 0;	 -- stores the original G value of the background/border before changing it
local storedB			= 0;	 -- stores the original B value of the background/border before changing it
local storedA			= 0;	 -- stores the original A value of the background/border before changing it

local BSSmiley			= 0;	 -- Smiley graphic object

-- /***********************************************
--   Bomb Sweeper Tables
--  *********************/

local BSGlyph			= {};	-- Graphics array of buttons
local BSGameGrid		= {};	-- Game grid data
local BSGameGridExtra		= {};	-- Game grid extra data (square clicked or not)

-- /***********************************************
--   Bomb Sweeper Defaults
--  *********************/

BSSettings = {
	["windowScale"] = 1.0;
	["backgroundR"] = 1;
	["backgroundG"] = 1;
	["backgroundB"] = 1;
	["backgroundA"] = 1;
	["borderR"]	= 1;
	["borderG"]	= 1;
	["borderB"]	= 1;
	["borderA"]	= 1;
	["tileGraphics"]= 1;
	["bombGraphics"]= 0;
	["roundWidth"]	= 10;
	["roundHeight"] = 10;
	["roundBombs"]	= 10;
	["chatAnnounce"] = true;
};

-- /***********************************************
--   Bomb Sweeper Functions
--  *********************/

-- /***********************************************
--   BS_OnLoad
--
--   Prepares the game for initial setup
--
--   accepts:	none
--   returns:	none
--  *********************/
function BS_OnLoad()

	-- Set our local variables
	local i, j = 0;
	local glyphIndex = 0;

	-- Register our events
	this:RegisterEvent("ADDON_LOADED");

	-- Create slash events
	SLASH_BOMBSWEEPER1 = "/bs";
	SLASH_BOMBSWEEPER2 = "/bsweep";
	SLASH_BOMBSWEEPER3 = "/bombsweeper";
	SlashCmdList["BOMBSWEEPER"] = BS_SlashHandler;
	
	-- Clamp the game to the screen
	local mainFrame = getglobal("BSFrameWindow")
	mainFrame:GetParent():SetClampedToScreen(true);

	-- Set the size of the window and center the frame (just incase)
	mainFrame:SetWidth(24 + 10 * 16);
	mainFrame:SetHeight(24 + 10 * 16)
	mainFrame:GetParent():SetWidth(24 + 10 * 16);
	mainFrame:GetParent():SetHeight(24 + 10 * 16 + BS_HEIGHT_TITLEBAR + BS_HEIGHT_FOOTERBAR);
	mainFrame:GetParent():ClearAllPoints();
	mainFrame:GetParent():SetPoint("CENTER", UIParent, "CENTER", 0, 0); 

	-- Setup our scale slider in the BSScale frame
	local slider = getglobal("BSScaleSlider");
	local string = getglobal("BSScaleSliderText");
	local thumb = getglobal("BSScaleSliderThumb");

	getglobal("BSScaleSliderThumb"):Show();
	getglobal("BSScaleSliderText"):SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
	getglobal("BSScaleSliderLow"):SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
	getglobal("BSScaleSliderHigh"):SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);

	slider:SetMinMaxValues(0.5, 1.5);
	slider:SetValueStep(0.05);
	slider:SetValue(1.0);
	string:SetText("Window Scale");

	-- Run through all of the graphic squares used for the game and create them
	for j = 1, BS_MAXROWS do
		for i = 1, BS_MAXCOLUMNS do

			-- Compute the graphic's index
			glyphIndex = i + (j - 1) * BS_MAXCOLUMNS;
			
			-- Build the new frame
			BSGlyph[glyphIndex] = BS_createFrame("Button", "BSGlyph" .. glyphIndex, mainFrame, 16, 16, "$addon\\art\\gameart1", true, glyphIndex);
			BS_setTexture(BSGlyph[glyphIndex], BS_HIDDEN, 4, 4);

			-- Position it in a grid
			if i == 1 and j == 1 then
				BSGlyph[1]:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", 12, - 12);
			else
				-- First in row
				if (i == 1) then
					BSGlyph[glyphIndex]:SetPoint("TOPLEFT", BSGlyph[glyphIndex - BS_MAXCOLUMNS], "BOTTOMLEFT", 0, 0);
	
				-- The rest in row
				else
					BSGlyph[glyphIndex]:SetPoint("TOPLEFT", BSGlyph[glyphIndex - 1], "TOPRIGHT", 0, 0);
				end
			end

			-- Register the clicking scripts
			BSGlyph[glyphIndex]:SetScript("OnClick", function() BS_checkSquare(this:GetID()) end);
			BSGlyph[glyphIndex]:RegisterForClicks("LeftButtonUp", "RightButtonUp");
		end
	end

	-- Create our smiley face object (shows at game over screen)
	BSSmiley = BS_createFrame("Button", "BSSmiley", mainFrame, 64, 64, "$addon\\art\\smiley", true, 0);
	BSSmiley:SetScript("OnClick", function() BS_createGame(BSSettings.roundWidth, BSSettings.roundHeight, BSSettings.roundBombs); end);
	BSSmiley:SetPoint("CENTER", mainFrame, "CENTER", 0, 0);
	BSSmiley:SetFrameLevel(BSSmiley:GetFrameLevel() + 2);
	BSSmiley:Show();

	-- Set the background of our value entry text boxes to transparent black
	getglobal("BSSettingsWidthValue"):SetBackdropColor(0,0,0,0.8);
	getglobal("BSSettingsHeightValue"):SetBackdropColor(0,0,0,0.8);
	getglobal("BSSettingsBombsValue"):SetBackdropColor(0,0,0,0.8);

end

-- /***********************************************
--   BS_Extract_NextParameter
--
--   Separates the parameters passed via a slash command,
--   and returns what we found
--
--   accepts:	chat message
--   returns:	command that was found, parameters that were found
--  *********************/
function BS_Extract_NextParameter(msg)
	
	-- Set our starting variables
	local params = msg;
	local command = params;

	-- Find the position of the parameters
	local index = string.find(command, " ");

	-- If there were parameters found, separate them from the command
	if ( index ) then
		command = string.sub(command, 1, index-1);
		params = string.sub(params, index+1);

	-- Otherwise, we just have a command
	else
		params = "";
	end
	
	-- Return what we found
	return string.lower(command), params;
end

-- /***********************************************
--   BS_SlashHandler
--
--   Handles all slash commands and reacts to them
--
--   accepts:	chat message
--   returns:	none
--  *********************/
function BS_SlashHandler(msg)

	-- Extract our command and parameters
	command, params = BS_Extract_NextParameter(msg);

	-- If the user wants us to show the frame, show it
	if (command=="show") then
		getglobal("BSFrame"):Show();
		getglobal("BSFrameMini"):Hide();
	
	-- If the user wants us to hide the frame, hide it
	elseif (command=="hide") then
		getglobal("BSFrame"):Hide();
		getglobal("BSFrameMini"):Hide();
	
	-- If the user doesn't want to do anything special, just 
	-- start up a new game and display the slash commands
	else
		-- Print out commands real quick...
		if BSSettings.chatAnnounce == true then
			BS_ChatPrint("Bomb Sweeper: Type \"/bs\" or \"/bs show\" to show the game if it's hidden", 0.3, 0.5, 0.92);
			BS_ChatPrint("Bomb Sweeper: Type \"/bs hide\" or hit the X button to hide the game", 0.3, 0.5, 0.92);
		end

		-- Show game
		getglobal("BSFrame"):Show();
		getglobal("BSFrameMini"):Hide();

	end
end

-- /***********************************************
--   BS_OnEvent
--
--   Handles all event messages
--
--   accepts:	none (event and args are passed)
--   returns:	none
--  *********************/
function BS_OnEvent()

	-- If we just loaded up...
	if (event == "ADDON_LOADED") and (arg1 == "BombSweeper") then

		-- Set our locals
		local i, j = 0;

		-- Check if we have saved variables. If not, make them
		if BSSettings.windowScale == nil then
			BSSettings.windowScale = 1.0;
		end
		if BSSettings.backgroundR == nil then
			BSSettings.backgroundR = 1;
			BSSettings.backgroundG = 1;
			BSSettings.backgroundB = 1;
			BSSettings.backgroundA = 1;
		end
		if BSSettings.borderR == nil then
			BSSettings.borderR = 1;
			BSSettings.borderG = 1;
			BSSettings.borderB = 1;
			BSSettings.borderA = 1;
		end
		if BSSettings.tileGraphics == nil then
			BSSettings.tileGraphics = 1;
		end
		if BSSettings.bombGraphics == nil then
			BSSettings.bombGraphics = 0;
		end
		if BSSettings.roundWidth == nil then
			BSSettings.roundWidth = 10;
			BSSettings.roundHeight = 10;
			BSSettings.roundBombs = 10;
		end
		if BSSettings.chatAnnounce == nil then
			BSSettings.chatAnnounce = 1.0;
		end
		if BSSettings.showTooltips == nil then
			BSSettings.showTooltips = true;
		end

		-- After loading the settings...

		-- Print our loaded message
		if BSSettings.chatAnnounce == true then
			BS_ChatPrint("Bomb Sweeper: Version 1.0 Loaded! Type \"/bs\" or \"/bsweep\" or \"/bombsweeper\" to play!", 0.3, 0.5, 0.92);
		end

		-- If the tileset that was loaded was different from the defaults that were loaded, update
		-- the tiles with the new graphics
		if BSSettings.tileGraphics ~= 1 then
			for j = 1, BS_MAXROWS do
				for i = 1, BS_MAXCOLUMNS do
					glyphIndex = i + (j - 1) * BS_MAXCOLUMNS;
					BSGlyph[glyphIndex]:SetNormalTexture(ADDON_PATH .. "\\art\\gameart" .. (BSSettings.tileGraphics))
				end
			end
		end

		-- Make sure we don't have incorrect data for our game which could CRASH our game
		BS_validateGameSettings();
		
		-- Create a game
		BS_createGame(BSSettings.roundWidth, BSSettings.roundHeight, BSSettings.roundBombs);

		-- Change the window scale if need be
		getglobal("BSScaleSlider"):SetValue(BSSettings.windowScale);
		getglobal("BSFrame"):SetScale(BSSettings.windowScale);

		-- Set the color of the background and border
		BS_updateSkinColor(BS_ALL);

		-- Create our Credits screen
		getglobal("BSCreditsFrameContents"):SetText(
		"|cFFFFFFFFCode and Modified Artwork by:\n" ..
		"|rMoongaze\n(Realm: Twisting Nether)\n\n"..
		"|cFFFFFFFFSpecial Thanks:\n\n" ..
		"|cFFFFFFFFRobert Donner and Curt Johnson\n" ..
		"|rCreators of the Windows Minesweeper game, based on the game Cube from 1973 ^_^\n\n" ..
		"|cFFFFFFFFBlizzard Entertainment, Inc.\n" ..
		"|rWithout WOW, this mod would not be available. Some artwork for this mod is also based upon art found within WOW.\n\n" ..
		"|cFFFFFFFFTechnetium\n" ..
		"|rResponsible for the fantastic TGAtoBLP2 and BLPtoTGA2 programs, helping to make all art included with this mod small and easy to bundle." );

	end
end

-- /***********************************************
--   BS_createFrame
--
--   Creates a frame based on parameters passed
--
--   accepts:	frame type, frame name, frame's parent, width and height of the frame, the texture file to
--		use, if it will use mouse input, and an ID for the frame
--   returns:	a frame
--  *********************/
function BS_createFrame(frameType, frameName, frameParent, width, height, textureFile, enableMouse, ID)

	-- create a temp frame
	local tempFrame = 0;

	-- Parse the texture file for the "$addon" string and replace it with the addon's path
	textureFile = string.gsub(textureFile, "$addon", ADDON_PATH);

	-- Create a frame with the details provided
	tempFrame = CreateFrame(frameType, frameName, frameParent);
	tempFrame:SetWidth(width);
	tempFrame:SetHeight(height);
	tempFrame:EnableMouse(enableMouse);
	tempFrame:SetID(ID);

	-- If the frame was a button, load the texture.
	if frameType == "Button" then
		tempFrame:SetNormalTexture(textureFile);
	end

	-- Return our new frame
	return tempFrame;

end

-- /***********************************************
--   BS_setTexture
--
--   Easy way to set a texture via a texture index
--
--   accepts:	object to modify, the texture index to use, number of textures found on the
--		horizontal, number of textures found on the vertical (i.e., picture cut into 4 parts
--		would usually be 2x2.
--   returns:	none
--  *********************/
function BS_setTexture(objectFrame, textureIndex, horizontalTextures, verticalTextures)

	-- Set our local variables
	local xLoc, yLoc, xSize, ySize = 0

	-- Get the offset for the texture to display
	yLoc = math.floor((textureIndex - 1) / horizontalTextures);
	xLoc = textureIndex - yLoc * horizontalTextures - 1;
	xSize = 1 / horizontalTextures;
	ySize = 1 / verticalTextures;

	-- Set the texture
	objectFrame:GetNormalTexture():SetTexCoord(xLoc * xSize, (xLoc + 1) * xSize, yLoc * ySize, (yLoc + 1) * ySize);
end

-- /***********************************************
--   BS_createGame
--
--   Creates a new game.
--
--   accepts:	width and height of game grid, total bombs used
--   returns:	none
--  *********************/
function BS_createGame (width, height, totalBombs)

	-- As usual, set our local variables
	local j, i, xLoc, yLoc, randNum, bombIndex = 0;
	local possibleCoords = {};

	-- Hide our smiley face, if it's around
	BSSmiley:Hide();

	-- Make sure we have a width, height, and bomb count that are in bounds. If not, set them.
	if width < 9 then
		width = 9;
	end
	if width > BS_MAXCOLUMNS then
		width = BS_MAXCOLUMNS;
	end
	if height < 9 then
		height = 9;
	end
	if height > BS_MAXROWS then
		height = BS_MAXROWS;
	end
	if totalBombs < 10 then
		totalBombs = 10;
	end
	if totalBombs > (width - 1) * (height - 1) then
		totalBombs = (width - 1) * (height - 1);
	end

	-- Store the game data
	BS_roundWidth = width
	BS_roundHeight = height
	BS_roundBombs = totalBombs
	BS_roundOver = false;

	-- Reset all graphics to "hidden tile" mode and hide the graphics
	for j = 1, BS_MAXROWS do
		for i = 1, BS_MAXCOLUMNS do
			BS_setTexture(BSGlyph[i + (j - 1) * BS_MAXCOLUMNS], BS_HIDDEN, 4, 4);
			BSGlyph[i + (j - 1) * BS_MAXCOLUMNS]:Hide();
		end
	end

	-- Resize the game window to match the size of our game grid
	getglobal("BSFrameWindow"):SetWidth(24 + width * 16);
	getglobal("BSFrameWindow"):SetHeight(24 + height * 16)
	getglobal("BSFrameWindow"):GetParent():SetWidth(24 + width * 16);
	getglobal("BSFrameWindow"):GetParent():SetHeight(24 + height * 16 + BS_HEIGHT_TITLEBAR + BS_HEIGHT_FOOTERBAR);

	-- Create a table full of possible bomb locations, clear the game grid, clear
	-- the extra grid, and show the graphics we're using
	BSGameGrid = {}
	BSGameGridExtra = {}
	for j = 1, height do
		BSGameGrid[j] = {}
		BSGameGridExtra[j] = {}
		for i = 1, width do
			BSGameGrid[j][i] = BS_EMPTY;
			BSGameGridExtra[j][i] = BS_EMPTY;
			possibleCoords[i + (j - 1) * width] = i + (j - 1) * width;
			BSGlyph[i + (j - 1) * BS_MAXCOLUMNS]:Show();
		end
	end

	-- set the randomizer to a new random seed
	-- ========================================================================
	-- HOWEVER! If the "Sweep" mine sweeper mod from Vallerius (located
	-- at: http://www.curse-gaming.com/en/wow/addons-1630-1-sweep.html) is
	-- loaded, we have to skip this line of code because there is a BAD BUG
	-- in their mod that basically sets the math.randomseed FUNCTION to NOTHING...
	-- ========================================================================
	if getglobal("Sweep_Frame") == nil then
		math.randomseed(math.random(0,214748364)+(GetTime()*1000));
	end

	-- Grab random bomb locations and dump them into the game grid
	for bombIndex = 1, totalBombs do
		randNum = math.random(1, table.getn(possibleCoords));
		yLoc = math.floor((possibleCoords[randNum] - 1) / width) + 1;
		xLoc = possibleCoords[randNum] - (yLoc - 1) * width;
		BSGameGrid[yLoc][xLoc] = BS_BOMB + BSSettings.bombGraphics;
		table.remove(possibleCoords, randNum);
		for j = -1, 1 do
			for i = -1, 1 do
				BS_markSection(xLoc + i, yLoc + j);
			end
		end
	end
end

-- /***********************************************
--   BS_markSection
--
--   Marks the game grid if a bomb doesn't exist in it's location. This is used
--   to help build the numbers around the bombs.
--
--   accepts:	x and y coordinate to check
--   returns:	none
--  *********************/
function BS_markSection(x, y)

	-- If we're out of bounds, leave now
	if x < 1 or x > BS_roundWidth or y < 1 or y > BS_roundHeight then
		return
	end
	
	-- Increase the grid's marker by one, if it's not a bomb
	if BSGameGrid[y][x] < BS_BOMB then
		BSGameGrid[y][x] = BSGameGrid[y][x] + 1;
	end
end

-- /***********************************************
--   BS_checkSquare
--
--   Checks the square to see if it was a bomb, or if we can clear it
--
--   accepts:	ID of the square to check
--   returns:	none
--  *********************/
function BS_checkSquare(squareID)

	-- if the game is ended, stop now
	if BS_roundOver == true then
		return;
	end

	-- 4, 8, 15, 16, 24, 42
	-- ^_^
	local x, y, bombsFound = 0;

	-- Calculate the x and y coordinate of the button on the game grid
	y = math.floor((squareID - 1) / BS_MAXCOLUMNS) + 1;
	x = squareID - (y - 1) * BS_MAXCOLUMNS;

	-- Before we do anything, let's see if the square was already revealed. If it has been, leave now.
	if BSGameGridExtra[y][x] == -1 then
		return
	end

	-- Check to see if the shift key is down OR the right mouse button was pressed. If it is, do something a little different
	if ( IsShiftKeyDown() or arg1 == "RightButton") then

		-- Add to our counter of markings. If we go past the two marking types, return to blank
		BSGameGridExtra[y][x] = BSGameGridExtra[y][x] + 1;
		if BSGameGridExtra[y][x] > 2 then
			BSGameGridExtra[y][x] = 0;
		end

		-- Show the type of mark we have on the square
		BS_setTexture(BSGlyph[squareID], BSGameGridExtra[y][x] + BS_HIDDEN, 4, 4);

		-- Check if we won
		bombsFound = 0;
		for y = 1, BS_roundHeight do
			for x = 1, BS_roundWidth do
				
				-- See if a bomb was marked (all must be marked to win). If so, count it as found.
				if BSGameGridExtra[y][x] == 1 and BSGameGrid[y][x] == (BS_BOMB + BSSettings.bombGraphics) then
					bombsFound = bombsFound + 1;
				end
			end
		end
		
		-- If all the bombs were found, end the game and show our happy face ^_^
		if bombsFound == BS_roundBombs then
			BS_roundOver = true;
			BS_setTexture(BSSmiley, 1, 2, 2);
			BSSmiley:SetAlpha(0.4);
			BSSmiley:Show();
		end
	
	-- Since the shift key was not held, we simply reveal the square and push through
	else

		-- if the grid has nothing (i.e., is blank), travel through the grid and
		-- clear all other clear blocks next to this one
		if BSGameGrid[y][x] == 0 then
			BS_clearBlocks(x, y);
	
		-- Otherwise, it's time to show what we hit...
		else
			BS_setTexture(BSGlyph[squareID], BSGameGrid[y][x], 4, 4);

			-- Notate that we revealed the square
			BSGameGridExtra[y][x] = -1

			-- Um, did we hit a bomb?
			if BSGameGrid[y][x] == BS_BOMB + BSSettings.bombGraphics then

				-- Yup ... BOOM! End the round, show all the bombs, show our sad face and end the game
				BS_roundOver = true;

				for y = 1, BS_roundHeight do
					for x = 1, BS_roundWidth do
						if BSGameGrid[y][x] == (BS_BOMB + BSSettings.bombGraphics) then
							BS_setTexture(BSGlyph[x + (y - 1) * BS_MAXCOLUMNS], BSGameGrid[y][x], 4, 4);
						end
					end
				end

				BS_setTexture(BSSmiley, 2, 2, 2);
				BSSmiley:SetAlpha(0.4);
				BSSmiley:Show();
			end
		end
	end

end

-- /***********************************************
--   BS_clearBlocks
--
--   Clears all the blocks that don't have a number in the specified square
--
--   accepts:	x and y coordinate to clear
--   returns:	none
--  *********************/
function BS_clearBlocks(x, y)

	-- If we are out of bounds, leave now
	if x < 1 or x > BS_roundWidth or y < 1 or y > BS_roundHeight then
		return
	end

	-- Mark that we're uncovered this area
	BSGameGridExtra[y][x] = -1;

	-- If the square holds nothing, set it as revealed and recursively trace
	-- all other clear blocks next to it
	if BSGameGrid[y][x] == 0 then
		BSGameGrid[y][x] = BS_REVEALED;
		BS_clearBlocks(x - 1, y    );
		BS_clearBlocks(x - 1, y - 1);
		BS_clearBlocks(x    , y - 1);
		BS_clearBlocks(x + 1, y - 1);
		BS_clearBlocks(x + 1, y    );
		BS_clearBlocks(x + 1, y + 1);
		BS_clearBlocks(x    , y + 1);
		BS_clearBlocks(x - 1, y + 1);
	end

	-- Update the game graphics to show our discovery
	BS_setTexture(BSGlyph[x + (y - 1) * BS_MAXCOLUMNS], BSGameGrid[y][x], 4, 4);
end

-- /***********************************************
--   BS_ChatPrint
--
--   Drops a line into the chat frame in a specified color
--
--   accepts:	text to write, red, green, and blue values (0.0 - 1.0)
--   returns:	none
--  *********************/
function BS_ChatPrint(text, red, green, blue)	
	DEFAULT_CHAT_FRAME:AddMessage(text, red, green, blue);
end

-- /***********************************************
--   BSDropDown_OnLoad
--
--   Sets up our drop down menu system
--
--   accepts:	none (frame is automatically passed)
--   returns:	none
--  *********************/
function BSDropDown_OnLoad()

	-- Set our menu system up and set sizes
	UIDropDownMenu_Initialize(getglobal("BSDropDown"), BSDropDown_Initialize, "MENU");
	UIDropDownMenu_SetButtonWidth(50);
	UIDropDownMenu_SetWidth(50);

end

-- /***********************************************
--   BSDropDown_Initialize
--
--   Prepare our menu system and the segments inside
--
--   accepts:	none
--   returns:	none
--  *********************/
function BSDropDown_Initialize()

	-- Our table that stores the menu info per item
	local info = {};

	-- Set up level 2 of our menu system
	if ( UIDROPDOWNMENU_MENU_LEVEL == 3 ) then

		-- Setup the Tile Graphics submenu
		if ( UIDROPDOWNMENU_MENU_VALUE == "Tile Graphics" ) then

			-- Create item [checked item] named: "Default"
			info = {};
			info.text = "Default";
			info.checked = nil;
			if ( BSSettings.tileGraphics == 1 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleTileGraphics(1) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			-- Create item [checked item] named: "Gems"
			info = {};
			info.text = "Gems";
			info.checked = nil;
			if ( BSSettings.tileGraphics == 2 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleTileGraphics(2) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			-- Create item [checked item] named: "Old School"
			info = {};
			info.text = "Old School";
			info.checked = nil;
			if ( BSSettings.tileGraphics == 3 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleTileGraphics(3) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			return;
		end

		-- Setup the Bomb Graphics submenu
		if ( UIDROPDOWNMENU_MENU_VALUE == "Bomb Graphics" ) then

			-- Create item [checked item] named: "Default Mine"
			info = {};
			info.text = "Default Mine";
			info.checked = nil;
			if ( BSSettings.bombGraphics == 0 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleBombGraphics(0) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			-- Create item [checked item] named: "Happy Bomb"
			info = {};
			info.text = "Happy Bomb";
			info.checked = nil;
			if ( BSSettings.bombGraphics == 1 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleBombGraphics(1) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			-- Create item [checked item] named: "Dynamite"
			info = {};
			info.text = "Dynamite";
			info.checked = nil;
			if ( BSSettings.bombGraphics == 2 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleBombGraphics(2) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			-- Create item [checked item] named: "Spikey Bomb"
			info = {};
			info.text = "Spikey Bomb";
			info.checked = nil;
			if ( BSSettings.bombGraphics == 3 ) then
				info.checked = 1;
			end
			info.func = function() BS_toggleBombGraphics(3) end;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

			return;
		end

	end

	-- Set up level 2 of our menu system
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then

		-- Create item [title] named: "Graphics"
		info = {};
		info.text = "Graphics";
		info.func = nil;
		info.notCheckable = 1;
		info.isTitle = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [function item] named: "Change Window Scale"
		info = {};
		info.text = "Change Window Scale";
		info.func =	function()
					--BSDropDown:Hide();
					getglobal("BSFrame"):ClearAllPoints();
					getglobal("BSFrame"):SetPoint("CENTER", UIParent, "CENTER", 0, 0); 
					BSScale:Show()
				end;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [spacer] named: ""
		info = {};
		info.disabled = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [expanded menu] named: "Tile Graphics"
		info = {};
		info.text = "Tile Graphics";
		info.hasArrow = 1;
		info.func = nil;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [expanded menu] named: "Bomb Graphics"
		info = {};
		info.text = "Bomb Graphics";
		info.hasArrow = 1;
		info.func = nil;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [spacer] named: ""
		info = {};
		info.disabled = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [color wheel item] named: "Background Color"
		info = {};
		info.text = "Background Color";
		info.hasColorSwatch = 1;
		info.r = BSSettings.backgroundR;
		info.g = BSSettings.backgroundG;
		info.b = BSSettings.backgroundB;
		info.hasOpacity = 1;
		info.opacity = BSSettings.backgroundA;
		info.swatchFunc = BS_changeBackgroundTint;
		info.opacityFunc = BS_changeBackgroundTint;
		info.func =	function ()
					storedR = BSSettings.backgroundR;
					storedG = BSSettings.backgroundG;
					storedB = BSSettings.backgroundB;
					storedA = BSSettings.backgroundA;
					UIDropDownMenuButton_OpenColorPicker();
				end;
		info.cancelFunc = function () BS_changeBackgroundTint(storedR, storedG, storedB, storedA); end; 
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [function item] named: "Reset Background Color"
		info = {};
		info.text = "Default Background Color";
		info.func = function () BS_changeBackgroundTint(1, 1, 1, 1); end;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [spacer] named: ""
		info = {};
		info.disabled = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [color wheel item] named: "Border Color"
		info = {};
		info.text = "Border Color";
		info.hasColorSwatch = 1;
		info.r = BSSettings.borderR;
		info.g = BSSettings.borderG;
		info.b = BSSettings.borderB;
		info.hasOpacity = 1;
		info.opacity = BSSettings.borderA;
		info.swatchFunc = BS_changeBorderTint;
		info.opacityFunc = BS_changeBorderTint;
		info.func =	function ()
					storedR = BSSettings.borderR;
					storedG = BSSettings.borderG;
					storedB = BSSettings.borderB;
					storedA = BSSettings.borderA;
					UIDropDownMenuButton_OpenColorPicker();
				end;
		info.cancelFunc = function () BS_changeBorderTint(storedR, storedG, storedB, storedA); end; 
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [function item] named: "Reset Border Color"
		info = {};
		info.text = "Default Border Color";
		info.func = function () BS_changeBorderTint(1, 1, 1, 1); end;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [spacer] named: ""
		info = {};
		info.disabled = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [title] named: "Other"
		info = {};
		info.text = "Other";
		info.func = nil;
		info.notCheckable = 1;
		info.isTitle = 1;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [checked item] named: "Show Chat Messages"
		info = {};
		info.text = "Show Chat Messages";
		info.checked = 1;
		if ( BSSettings.chatAnnounce == false ) then
			info.checked = nil;
		end
		info.func = 
			function()
				if (BSSettings.chatAnnounce == true) then
					BSSettings.chatAnnounce = false;
				else
					BSSettings.chatAnnounce = true;
				end
			end;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		-- Create item [checked item] named: "Show Tooltips"
		info.text = "Show Tooltips";
		info.checked = 1;
		if ( BSSettings.showTooltips == false ) then
			info.checked = nil;
		end
		info.func = 
			function()
				if (BSSettings.showTooltips == true) then
					BSSettings.showTooltips = false;
				else
					BSSettings.showTooltips = true;
				end
			end;
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

		return;
	end

	-- Create item [function item] named: "New Game"
	info = {};
	info.text = "New Game";
	info.func =	function ()
				BS_createGame(BSSettings.roundWidth, BSSettings.roundHeight, BSSettings.roundBombs); 
			end;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	-- Create item [function item] named: "Game Setup"
	info = {};
	info.text = "Game Setup";
	info.func =	function ()
				BSFrameSettings:Show(); 
			end;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	-- Create item [expanded menu] named: "Options"
	info = {};
	info.text = "Options";
	info.hasArrow = 1;
	info.func = nil;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	-- Create item [expanded menu] named: "Credits"
	info = {};
	info.text = "Credits";
	info.func =	function () 
				getglobal("BSCreditsFrame"):Show();
			end;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	-- Create item [function item] named: "Hide Game"
	info = {};
	info.text = "Hide Game";
	info.func =	function () 
				getglobal("BSFrame"):Hide();
			end;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

end

-- /***********************************************
--   BSScale_SetDefault()
--
--   Sets the window scale to default
--
--   accepts:	none
--   returns:	none
--  *********************/
function BSScale_SetDefault()

	-- Set our defaults, update the interface, and save the values
	getglobal("BSScaleSlider"):SetValue(1.0);
	getglobal("BSFrame"):SetScale(1.0);
	BSSettings.windowScale = 1.0;

end

-- /***********************************************
--   BSScale_CancelScale()
--
--   Sets the window scale to back to what it was
--
--   accepts:	none
--   returns:	none
--  *********************/
function BSScale_CancelScale()

	-- Set our scale again, update the interface
	getglobal("BSScaleSlider"):SetValue(BSSettings.windowScale);
	getglobal("BSFrame"):SetScale(BSSettings.windowScale);

end

-- /***********************************************
--   BSScale_ChangeScale()
--
--   Sets the window scale to a new scale
--
--   accepts:	none
--   returns:	none
--  *********************/
function BSScale_ChangeScale()

	-- Set our scale, update the interface, save it
	BSSettings.windowScale = getglobal("BSScaleSlider"):GetValue();
	getglobal("BSFrame"):SetScale(BSSettings.windowScale);

end

-- /***********************************************
--   BS_changeBackgroundTint()
--
--   Changes the color of the background in the game
--
--   accepts:	none
--   returns:	none
--  *********************/
function BS_changeBackgroundTint(r, g, b, a)

	-- If we don't have values that were passed, we need to build them
	if r == nil then
		--Change the values to the color picker values
		r,g,b = ColorPickerFrame:GetColorRGB();
		a = getglobal("OpacitySliderFrame"):GetValue();
	end

	-- Set our new background settings
	BSSettings.backgroundR = r;
	BSSettings.backgroundG = g;
	BSSettings.backgroundB = b;
	BSSettings.backgroundA = a;
	
	-- Apply the tint to the background
	BS_updateSkinColor(BS_BACKGROUND);

end

-- /***********************************************
--   BS_changeBorderTint()
--
--   Changes the color of the border in the game
--
--   accepts:	none
--   returns:	none
--  *********************/
function BS_changeBorderTint(r, g, b, a)

	-- If we don't have values that were passed, we need to build them
	if r == nil then
		--Change the values to the color picker values
		r,g,b = ColorPickerFrame:GetColorRGB();
		a = getglobal("OpacitySliderFrame"):GetValue();
	end

	-- Set our new background settings
	BSSettings.borderR = r;
	BSSettings.borderG = g;
	BSSettings.borderB = b;
	BSSettings.borderA = a;
	
	-- Apply the tint to the borders
	BS_updateSkinColor(BS_BORDER);

end

-- /***********************************************
--   BS_toggleTileGraphics
--
--   Changes the tileset of the graphics displayed in the game
--
--   accepts:	tile set number
--   returns:	none
--  *********************/
function BS_toggleTileGraphics(tileSet)

	-- Set our local variables
	local i, j = 0;

	-- If the tile set is different from the one already loaded, we make our changes
	if BSSettings.tileGraphics ~= tileSet then

		-- Travel the grid and update all textures
		for j = 1, BS_MAXROWS do
			for i = 1, BS_MAXCOLUMNS do
				glyphIndex = i + (j - 1) * BS_MAXCOLUMNS;
				BSGlyph[glyphIndex]:SetNormalTexture(ADDON_PATH .. "\\art\\gameart" .. tileSet)
			end
		end
		
		-- Set the tile set to the new tile set
		BSSettings.tileGraphics = tileSet;
	end
end

-- /***********************************************
--   BS_toggleBombGraphics
--
--   Changes the bomb graphics displayed in the game
--
--   accepts:	bomb set number
--   returns:	none
--  *********************/
function BS_toggleBombGraphics(bombSet)

	-- If the bomb set is already being used, let's just leave.
	if bombSet == BSSettings.bombGraphics then
		return
	end

	-- Set our locals
	local i, j = 0;

	-- Travel and update bomb data stored in our game grid
	for j = 1, BS_roundHeight do
		for i = 1, BS_roundWidth do
			if BSGameGrid[j][i] == (BS_BOMB + BSSettings.bombGraphics) then
				BSGameGrid[j][i] = BS_BOMB + bombSet;
			end
		end
	end

	-- Set our new bomb set
	BSSettings.bombGraphics = bombSet;

end

-- /***********************************************
--   BS_validateGameSettings
--
--   Checks to make sure we don't use invalid numbers for our settings
--
--   accepts:	none
--   returns:	none
--  *********************/
function BS_validateGameSettings()

	-- Run through each setting value and make sure it's in range
	if BSSettings.roundWidth < 9 then 
		BSSettings.roundWidth = 9; 
	end
	if BSSettings.roundWidth > BS_MAXCOLUMNS then 
		BSSettings.roundWidth = BS_MAXCOLUMNS; 
	end
	if BSSettings.roundHeight < 9 then 
		BSSettings.roundHeight = 9;
	end
	if BSSettings.roundHeight > BS_MAXROWS then 
		BSSettings.roundHeight = BS_MAXROWS;
	end
	if BSSettings.roundBombs < 9 then 
		BSSettings.roundBombs = 9;
	end
	if BSSettings.roundBombs > (BSSettings.roundWidth - 1) * (BSSettings.roundHeight - 1) then
		BSSettings.roundBombs = (BSSettings.roundWidth - 1) * (BSSettings.roundHeight - 1);
	end

end

-- /***********************************************
--   BS_updateSkinColor
--
--   Changes the color of the borders and background of the game
--
--   accepts:	update flag (BS_ALL, BS_BORDER, or BS_BACKGROUND)
--   returns:	none
--  *********************/
function BS_updateSkinColor(updateType)

	-- If we need to update the border or all items, update the borders
	if updateType == BS_BORDER or updateType == BS_ALL then
		getglobal("BSFrame"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
		getglobal("BSFrameWindow"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
		getglobal("BSFrameSettings"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
		getglobal("BSScale"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
		getglobal("BSSettingsWidthValue"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
		getglobal("BSSettingsHeightValue"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
		getglobal("BSSettingsBombsValue"):SetBackdropBorderColor(BSSettings.borderR, BSSettings.borderG, BSSettings.borderB, BSSettings.borderA);
	end

	-- If we need to update the background or all items, update the background
	if updateType == BS_BACKGROUND or updateType == BS_ALL then
		getglobal("BSFrame"):SetBackdropColor(BSSettings.backgroundR, BSSettings.backgroundG, BSSettings.backgroundB, BSSettings.backgroundA);
		getglobal("BSFrameSettings"):SetBackdropColor(BSSettings.backgroundR, BSSettings.backgroundG, BSSettings.backgroundB, BSSettings.backgroundA);
		getglobal("BSScale"):SetBackdropColor(BSSettings.backgroundR, BSSettings.backgroundG, BSSettings.backgroundB, BSSettings.backgroundA);
	end
end